#ifndef VECTOR_SPACE_SPARSE_H
#define VECTOR_SPACE_SPARSE_H 1

#include <set>
#include <vector>
#include <iostream>
#include <algorithm>
#include <cstdlib>

namespace AG
{
	template <typename VALUE_TYPE>
	class Point
	{
	public:

		// A pair
		//
		typedef typename std::pair<unsigned int, VALUE_TYPE> Pair;

		// Constructor
		//
		Point(void) : _sorted(false) {};
		~Point() {};

		// push_back
		//
		inline void push_back(const unsigned int i, const VALUE_TYPE d) 
		{ 
			_v.push_back(std::make_pair(i, d));  
			_sorted = false;
		}

		// sort
		//
		void sort(void) 
		{ 
			if (!_sorted)
				std::sort(_v.begin(), _v.end(), sort_pred()); 
			_sorted = true;
		}

		// setSorted
		// 
		void setSorted(void) { _sorted = true; }

		// getSorted
		// 
		bool getSorted(void) const { return _sorted;}


		// size
		//
		size_t size() { return _v.size(); }

		// clear
		void clear() { _v.clear(); }

		// access read only
		//
		const Pair & operator [] (const unsigned int i) const { return _v[i]; }

	private:

		// sort predicate
		//
		struct sort_pred
		{
			bool operator () (const Pair& left, const Pair& right)
			{
				return left.first < right.first;
			}
		};

		// vector of pairs (position, value)
		//
		std::vector<Pair> _v;
		bool _sorted;

	// Dump a point
	//
	template <typename VALUE_TYPE>
	friend std::ostream& operator << (std::ostream& os, const Point<VALUE_TYPE>& p);

	}; // class Point

	// Dump a point
	//
	template <typename VALUE_TYPE>
	std::ostream& operator << (std::ostream& os, const Point<VALUE_TYPE>& p)
	{
		unsigned int size = p._v.size();
		for (unsigned int i = 0; i < size; i++)
			os << " (" << p._v[i].first << ", " << p._v[i].second << ") ";
		return os;
	}

};

namespace AG
{
	template <typename VALUE_TYPE>
	class VectorSpace{

	public:
		VectorSpace(unsigned int numPoints, unsigned int maxNumFeatures, double maxSparseness, unsigned int maxValue) :
		  _numPoints(numPoints), _maxNumFeatures(maxNumFeatures), _maxSparseness(maxSparseness), _maxValue(maxValue)
		{
			_data.resize(numPoints);  // allocate the space needed 

			for (unsigned int i = 0 ; i < numPoints; i++)
			{
				generatePoint(i);
				std::cout << "Generated point " << i << std::endl;
			}
		};

	private:
		
		void generatePoint(unsigned int i) 
		{ 
			Point<VALUE_TYPE> p;
			unsigned int featureId, howManyFeatures;
			unsigned int value;

			howManyFeatures =  (int) (_maxNumFeatures * _maxSparseness);
			howManyFeatures = rand () % howManyFeatures;

			for (unsigned int j = 0 ; j < howManyFeatures; j++)
			{
				featureId = (int) (rand()  % _maxNumFeatures );
				value = (int) (rand() % _maxValue);
				p.push_back(featureId, value);
			}

			p.sort(); // sort the point before inserting in vectorSpace. just for improving performances
			_data[i] = p;

			std::cout << _data[i] << std::endl;
		};
		
		std::vector<Point<VALUE_TYPE> > _data;
		unsigned int _numPoints;
		unsigned int _maxNumFeatures;
		double _maxSparseness;
		unsigned int _maxValue;
	};

};


#endif